@@ -265,4 +265,48 @@ register("OX_Configuration_Loaded_Complete", function() {
         }
     };
     
+    // OAuth autocreate hook
+    temporary.services.provide("com.openexchange.oauth.autocreate", {
+        init: function(container) {
+            this.checkbox = new ox.UI.CheckBox(_("Create a Social Messaging account"));
+            this.checkbox.default_value = true;
+            container.addWidget(this.checkbox, "");
+        },
+        update: function(oauthid) {
+            var self = this;
+            messagingServices.get(function(services) {
+                for (var i in services) {
+                    var form = services[i].formDescription;
+                    if (form.length == 1 && form[0].widget == "oauthAccount" &&
+                        form[0].options.type == oauthid)
+                    {
+                        self.service = services[i];
+                        self.checkbox.show();
+                        return;
+                    }
+                }
+                self.checkbox.hide();
+            });
+        },
+        save: function(data) {
+            if (!this.checkbox.get()) return;
+            var config = {};
+            config[this.service.formDescription[0].name] = data.id;
+            ox.JSON.put(AjaxRoot + "/messaging/account/?action=new&session=" +
+                session, dataToJson({
+                    configuration: config,
+                    displayName: data.displayName,
+                    messagingService: this.service.id
+                }), function() {
+                    ox.api.folder.getSubFolders({
+                        folder: 1,
+                        cache: false,
+                        success: function() {
+                            ox.api.folder.dispatcher.trigger("modify");
+                        }
+                    });
+                });
+        }
+    });
+    
 });
\ No newline at end of file
