@@ -711,7 +711,7 @@ function setMessageText(message) {
         ih.open();
         ih.writeln('<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN"'
                 + '     "http://www.w3.org/TR/html4/strict.dtd">'
-                + '<html><head><title>' + (message.subject || "") + '</title>'
+                + '<html><head><title>' + (escapeHTML(message.subject || "")) + '</title>'
                 + '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">'
                 + '<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=IE7">'
                 + '<base href="' + "[protocol]://[hostname][path]newInfoItemHidden.html".format()  + '">'
