@@ -2210,7 +2210,7 @@ function sendMailToRecipientMail(recipient) {
     if(recipient && recipient != "") {
         ox.api.mail.compose({
             data: {
-                to: [[recipient]]
+                to: [typeof recipient === "string" ? [recipient] : recipient]
             }
         });
     }
