@@ -417,11 +417,17 @@ newMailOnload = function () {
      * attach autocomplete to recipient dialogs if enabled in the user options
      */
     if (configGetKey("gui.mail.autocomplete")) {
-        var dummy; // for jslint
-        dummy = new AutoComplete($('to'), $('AutoCompleteContent'), 25, configGetKey("minimumSearchCharacters"));
-        dummy = new AutoComplete($('cc'), $('AutoCompleteContent'), 25, configGetKey("minimumSearchCharacters"));
-        dummy = new AutoComplete($('bcc'), $('AutoCompleteContent'), 25, configGetKey("minimumSearchCharacters"));
-        dummy = null;
+        var ac; // for jslint
+        ac = new AutoComplete($('to'), $('AutoCompleteContent'), 25, configGetKey("minimumSearchCharacters"));
+        ac.onSelect = onAutoComplete;
+        ac = new AutoComplete($('cc'), $('AutoCompleteContent'), 25, configGetKey("minimumSearchCharacters"));
+        ac.onSelect = onAutoComplete;
+        ac = new AutoComplete($('bcc'), $('AutoCompleteContent'), 25, configGetKey("minimumSearchCharacters"));
+        ac.onSelect = onAutoComplete;
+        ac = null;
+    }
+    function onAutoComplete(element, scrolltop) {
+        virtualInputs[this.oText.id].set(element.getAttribute("oval"));
     }
 
     // add simple context menu for attachments
@@ -522,12 +528,13 @@ finalizeGUI = function () {
 
         // adjust view states
         for (i in options) {
-            oMailFunctionsNew.showAdditionalHeaderField(i, options[i]);
+            oMailFunctionsNew.showAdditionalHeaderField(i, options[i] ||
+                oMailFunctionsNew.additionalHeaderFieldIsVisible(i));
         }
 
         resizeEvents.register("Resized", function () {
             // ugly hack to fix bug #18051.
-            // check to get rif of the toolbar resize race condition,
+            // check to get rid of the toolbar resize race condition,
             if (oMailFunctionsNew.attachObj.length &&
                     jQuery("#mailsplitbottom").css("bottom") === "0px") {
                 // force a redraw of the attachment panel if it seems that the
@@ -726,6 +733,7 @@ function joinLines(lines) {
 
 // -----------------------------------------------------------------------------
 var accountAliases = {}; // aliases mapping
+var virtualInputs = {}; // ID-based mapping of virtual inputs
 initMessageHeader = function () {
 
     var defaultName = quotePersonalAddr(corewindow.getPersonalFullName()) + " ",
@@ -795,7 +803,7 @@ initMessageHeader = function () {
     jQuery.extend(accountAliases, aliases);
 
     var scrollBar = ObjectToArray(accountAliases).length > 5 ? 6 : null;
-    senderAddress = window.heinz = new ComboBox3(window, "mail.fromCombo", "25em", 0,
+    senderAddress = new ComboBox3(window, "mail.fromCombo", "25em", 0,
             true, "cb_mailComboChanged", scrollBar, { type: "sender" });
     // add to combo-box
     for (var i in accountAliases) {
@@ -811,7 +819,12 @@ initMessageHeader = function () {
 
     // set the default selected address
     senderAddress.setKey(primaryAddress, true);
-
+    
+    // configure virtual inputs for to, cc and bcc
+    jQuery(".virtual-input > .input > input").each(function (_, input) {
+        virtualInputs[input.id] = new VirtualInput(input);
+    });
+    
     priority = new ComboBox3(window, "mail.priorityCombo", "13em", 1, true, "cb_mailComboChanged", 3, { type: "priority" });
     priority.addElement(_("Low"), "4");
     priority.addElement(_("Normal"), "3");
@@ -852,6 +865,398 @@ initMessageHeader = function () {
     }
 };
 
+function VirtualInput(input) {
+    var self = this;
+    this.input = input;
+    this.storage = new Storage(0, []);
+    this.selection = new Selection();
+    this.selection.events.register("Selected",
+        function() { self.updateStyles(); });
+    this.selection.setStorage(this.storage);
+    this.container = input.parentNode.parentNode;
+    this.marker = jQuery(".marker", this.container).get(0);
+    this.value = "";
+    this.terminated = true;
+    jQuery(input).on({
+        "keyup cut input": function(e) {
+            e.stopPropagation();
+            self.check(true);
+        },
+        "change paste blur": function(e) {
+            e.stopPropagation();
+            self.check(false);
+        },
+        keydown: function(e) {
+            e.stopPropagation();
+            if ((e.which == 37 || e.which == 8) && // arrow left or backspace
+                !e.shiftKey && !e.ctrlKey && !e.metaKey && 
+                e.target.selectionStart == 0 && e.target.selectionEnd == 0)
+            {
+                self.focus = self.storage.ids.length - 1;
+                self.selection.click(self.focus, e);
+                e.preventDefault();
+            }
+        },
+        focus: function(e) {
+            self.focus = -1;
+            self.selection.reset();
+        }
+    });
+    jQuery(this.container).on({
+        keydown: function(e) {
+            switch (e.which) {
+            case 13: // enter
+                if (e.metaKey || e.ctrlKey || e.shiftKey) return;
+                if (self.selection.count !== 1) break;
+                self.edit(self.selection.getSelected());
+                break;
+            case 32: // spacebar
+                self.selection.click(self.focus, e);
+                break;
+            case 8: // backspace
+            case 46: // delete
+            case 108: // numpad delete
+                if (e.metaKey || e.ctrlKey || e.shiftKey) return;
+                self.remove(self.selection.getSelected());
+                break;
+            case 37: // arrow left
+                if (self.focus > 0) self.focus--;
+                if (Mac ? e.metaKey : e.ctrlKey) {
+                    self.selection.events.trigger("Selected",
+                            self.selection.count);
+                } else {
+                    self.selection.click(self.focus, e);
+                }
+                break;
+            case 39: // arrow right
+                if (self.focus < self.storage.ids.length - 1) {
+                    self.focus++;
+                } else if (!e.metaKey && !e.ctrlKey && !e.shiftKey) {
+                    self.focus = -1;
+                    self.selection.reset();
+                    input.focus();
+                    break;
+                }
+                if (Mac ? e.metaKey : e.ctrlKey) {
+                    self.selection.events.trigger("Selected",
+                            self.selection.count);
+                } else {
+                    self.selection.click(self.focus, e);
+                }
+                break;
+            case 65: // A
+            case 97: // a
+                if (e.ctrlKey) {
+                    self.selection.select(0, self.storage.ids.length);
+                }
+                break;
+            default:
+                return;
+            }
+            e.stopPropagation();
+            e.preventDefault();
+        },
+        click: function(e) {
+            e.stopPropagation();
+            e.preventDefault();
+            self.focus = jQuery(e.currentTarget).index();
+            if (self.selection.get(self.focus) &&
+                !e.shiftKey && !e.ctrlKey && !e.metaKey)
+            {
+                self.edit([self.storage.ids[self.focus]]);
+            } else {
+                self.selection.click(self.focus, e);
+            }
+        },
+        dblclick: function(e) {
+            e.preventDefault();
+            e.stopPropagation();
+            self.edit([self.storage.ids[jQuery(e.currentTarget).index()]]);
+        }
+    }, ".address");
+//    if (VirtualInput.all.push(this) === 1) VirtualInput.loop();
+}
+
+VirtualInput.all = [];
+
+VirtualInput.loop = function() {
+    for (var i = 0; i < VirtualInput.all.length; i++) {
+        VirtualInput.all[i].check(true);
+    }
+    if (VirtualInput.all.length) setTimeout(VirtualInput.loop, 1000);
+};
+
+VirtualInput.prototype.check = function(terminated) {
+    var value = this.input.value;
+    // Ignore unchanged values, except to check for the last unterminated
+    // address when terminated changes to false.
+    if (this.value === value && (!this.terminated || terminated)) return;
+    this.terminated = terminated;
+    
+    setBeforeUnload(true);
+    
+    var index = this.parse(value, this.marker, terminated);
+    value = this.value = value.slice(index);
+    if (index === 0) return;
+    if (document.activeElement === this.input && this.input.selectionStart) {
+        var selStart = this.input.selectionStart,
+            selEnd = this.input.selectionEnd,
+            selDir = this.input.selectionDirection;
+        this.input.value = value;
+        this.input.setSelectionRange(Math.max(0, selStart - index),
+                                     Math.max(0, selEnd   - index), selDir);
+    } else {
+        this.input.value = value;
+    }
+    // Workaround for browser bug:
+    // The input field doesn't change its width when jumping to a new line
+    var parent = this.input.parentNode;
+    setTimeout(function() {
+        parent.style.overflow = "auto";
+        setTimeout(function() { parent.style.overflow = ""; }, 0);
+    }, 0);
+};
+
+VirtualInput.prototype.parse = function(value, marker, terminated, position) {
+    var index = 0, addrs = [], match;
+    emailRegExps.separator.lastIndex = 0;
+    while ((match = emailRegExps.separator.exec(value))) {
+        var address = trimStr(match[1]);
+        if (!match[2] && (terminated || !address)) break;
+        index = emailRegExps.separator.lastIndex;
+        if (!address) continue;
+        this.asyncUpdate();
+        var addr = new VirtualInput.Address(this, address);
+        this.container.insertBefore(addr.node, marker);
+        addrs.push(addr);
+    }
+    
+    var ids = [], i, j;
+    if (position === undefined) position = this.storage.ids.length;
+    for (i = 0; i < position; i++) ids.push([this.storage.ids[i]]);
+    for (j = 0; j < addrs.length; j++) ids.push(addrs[j]);
+    for (; i < this.storage.ids.length; i++) ids.push([this.storage.ids[i]]);
+    this.storage.update(0, ids, addrs);
+    return index;
+};
+
+VirtualInput.prototype.updateStyles = function() {
+    var self = this;
+    this.storage.newIterate(this.storage.ids, 0, function(i, addr) {
+        var node = addr.node, oldClass = String(node.className);
+        var newClass = oldClass.replace(/address(?: selected)?/,
+                self.selection.get(i) ? "address selected" : "address");
+        if (newClass !== oldClass) node.className = newClass;
+        if (i == self.focus) node.focus();
+    });
+};
+
+/**
+ * Processes asynchronous updates which may affect the layout.
+ * Call this method immediately before changing the layout.
+ * Multiple calls are automatically joined via setTimeout().
+ */
+VirtualInput.prototype.asyncUpdate = function() {
+    if (this.asyncUpdateTimer != null) return;
+    var self = this, height = this.container.offsetHeight;
+    this.asyncUpdateTimer = setTimeout(function() {
+        self.asyncUpdateTimer = null;
+        if (self.container.offsetHeight === height) return;
+        resizeSplit("newMailAttachmentDiv",
+            $("newMailAttachmentDiv").clientHeight);
+    }, 0);
+};
+
+VirtualInput.prototype.setFocus = function(index) {
+    this.focus = index;
+    this.selection.click(this.focus);
+};
+
+VirtualInput.prototype.edit = function(ids) {
+    this.focus = -1;
+    this.selection.reset();
+    var self = this;
+    this.storage.newIterate(ids, 0, function(i, addr) {
+        if (i < self.storage.ids.length - 1) {
+            addr.edit();
+        } else {
+            addr.edit(self.input);
+        }
+    });
+};
+
+VirtualInput.prototype.remove = function(ids, focus) {
+    if (!focus) {
+        var id;
+        if (this.focus >= 0) {
+            var removed = {};
+            for (var i = 0; i < ids.length; i++) {
+                removed[this.storage.indices[ids[i]]] = true;
+            }
+            for (var i = this.focus; removed[i]; i++);
+            id = this.storage.ids[i];
+        }
+    }
+    
+    this.asyncUpdate();
+    this.storage.newIterate(ids, 0, function(i, addr) { addr.remove(); });
+    this.storage.removeIDs(ids);
+    
+    if (focus) {
+        focus.focus();
+    } else if (id === undefined) {
+        try {
+            this.input.focus();
+        } catch (e) { /* stupid IE8 */ }
+    } else {
+        var self = this;
+        setTimeout(function() {
+            self.focus = self.storage.indices[id];
+            if (self.selection.count) {
+                self.selection.events.trigger("Selected", self.selection.count);
+            } else {
+                self.selection.click(self.focus);
+            }
+        }, 0);
+    }
+};
+
+VirtualInput.prototype.get = function() {
+    var s = [];
+    this.storage.newIterate(this.storage.ids, 0, function(i, addr) {
+        s.push(getFullAddress(addr.name, addr.address));
+    });
+    var last = trimStr(this.input.value);
+    if (last) s.push(last);
+    return s.join(", ");
+};
+
+VirtualInput.prototype.getArray = function() {
+    this.check(false);
+    var a = [];
+    this.storage.newIterate(this.storage.ids, 0, function(i, addr) {
+        a.push([quotePersonalAddr(addr.name), addr.address]);
+    });
+    var last = trimStr(this.input.value);
+    if (last) a.push([last, null]);
+    return a;
+};
+
+VirtualInput.prototype.set = function(value) {
+    this.input.value = value;
+    this.check(false);
+};
+
+VirtualInput.prototype.clear = function () { this.remove(this.storage.ids); };
+
+/**
+ * Callf a function for each parsed address.
+ * @param callback A function which is called with the index and
+ * the VirtualInput.Address object as parameters.
+ */
+VirtualInput.prototype.each = function(callback) {
+    this.storage.newIterate(this.storage.ids, 0, callback);
+};
+
+VirtualInput.Address = function(parent, address) {
+    var match = emailRegExps.email.exec(address);
+    if (match) {
+        this.name = trimStr(match[1]);
+        this.address = match[2] || match[3];
+    } else {
+        this.name = "";
+        this.address = address;
+    }
+    this.parent = parent;
+    this[0] = VirtualInput.Address.nextId++; // unique ID for Storage
+    var self = this;
+
+    if (this.name) this.name = unescapeDisplayName(this.name);
+    this.node = jQuery('<div class="address" tabindex="-1"></div>')
+        .text(this.name || this.address)
+        .append(jQuery('<div class="delete"></div>').click(function(e) {
+            e.stopPropagation();
+            self.parent.remove([self[0]]);
+        }));
+    if (IE < 10) {
+        this.node.on("selectstart", function () { return false; });
+    }
+    if (this.name) {
+        this.node.attr("title", getFullAddress(this.name, this.address));
+    }
+    this.node = this.node.get(0);
+    triggerEvent("newAddress", this);
+};
+
+VirtualInput.Address.nextId = 1;
+
+VirtualInput.Address.prototype.get = function() {
+    return getFullAddress(this.name, this.address);
+};
+
+VirtualInput.Address.prototype.remove = function() {
+    triggerEvent("deleteAddress", this);
+    this.node.parentNode.removeChild(this.node);
+};
+
+VirtualInput.Address.prototype.edit = function(input) {
+    var self = this;
+    var pos = jQuery(this.node).index();
+    if (input) {
+        input.value = this.get() + input.value;
+    } else {
+        var text = jQuery('<div>').text(this.get()).css({
+            "position": "absolute",
+            "visibility": "hidden"
+        }).appendTo(this.node);
+        var textWidth = text.width();
+        text.text(this.name || this.address);
+        textWidth = textWidth - text.width() + this.node.clientWidth - 8;
+        text.remove();
+        
+        input = jQuery('<input type="text" class="edit" tabindex="-1">')
+            .val(this.get())
+            .css("width", textWidth + "px")
+            .on("keydown", function(e) {
+                if (e.shiftKey || e.ctrlKey || e.metaKey) return;
+                switch (e.which) {
+                case 13: // ente
+                    done(0);
+                    break;
+                case 37: // arrow left
+                    var t = e.target;
+                    if (t.selectionStart != 0 || t.selectionEnd != 0) return;
+                    if (pos <= 0) return;
+                    done(-1);
+                    break;
+                case 39: // arrow right
+                    var t = e.target, L = t.value.length;
+                    if (t.selectionStart != L || t.selectionEnd != L) return;
+                    done(1);
+                    break;
+                default:
+                    return;
+                }
+                e.stopPropagation();
+                e.preventDefault();
+            })
+            .on("blur", done)
+            .get(0);
+        this.node.parentNode.insertBefore(input, this.node);
+    }
+    this.parent.remove([this[0]], input);
+    
+    function done(focus) {
+        jQuery(input).off("blur", done);
+        var value = input.value;
+        self.parent.parse(value, input, false, pos);
+        if (focus > 0) pos = jQuery(input).index() - 1;
+        input.parentNode.removeChild(input);
+        if (focus !== undefined) self.parent.setFocus(pos + focus);
+    }
+};
+
+
 cb_mailComboChanged = function (key, args) {
     // set reply-to address
     if (ox.api.config.get("ui.mail.replyTo.configurable") === true
@@ -994,12 +1399,24 @@ setMailFields = function (mailobject) {
 };
 
 setRecipients = function (to, field) {
-    if (to && to.length && $(field)) {
-        $(field).value = corewindow.getAdressString(to, false);
-        if (field !== "to" && trimStr($(field).value).length !== 0) {
-            // don't use trigger event because we won't store the settings!
-            oMailFunctionsNew.showAdditionalHeaderField(field, true);
+    if (typeof to !== "string" && to &&
+            (to.length != 1 || to[0] && to[0].length > 1))
+    {
+        var value = jQuery.map(to, function(a) {
+            return getFullAddress(a[0] ? unescapeDisplayName(a[0]) : "", a[1]);
+        }).join(", ");
+    } else {
+        if (debug) {
+            console.warn(field, "is not separated into name and address:", to);
+            if (console.trace) console.trace();
         }
+        var value = corewindow.getAdressString(to, false);
+    }
+    virtualInputs[field].clear();
+    virtualInputs[field].set(value);
+    if (field !== "to" && trimStr(value)) {
+        // don't use trigger event because we won't store the settings!
+        oMailFunctionsNew.showAdditionalHeaderField(field, true);
     }
 };
 
@@ -1166,7 +1583,7 @@ MailFunctions.prototype = {
             } else if (action === "reply" || action === "replyall") {
                 Self.oo.sendtype = 1;
             } else if (action === "draft") {
-                Self.oo.sendtype = 4;
+                Self.oo.sendtype = param ? 3 : 4;
             }
             Self.sAction = AjaxRoot + "/mail?action=new&session=" + corewindow.session;
             var sTargetParam = 'mailSendCB';
@@ -1355,9 +1772,9 @@ MailFunctions.prototype = {
                 this.oo.reply_to = reply_to;
             }
         }
-        this.oo.to = $("to").value;
-        this.oo.cc = $("cc").value;
-        this.oo.bcc = $("bcc").value;
+        this.oo.to = virtualInputs.to.getArray();
+        this.oo.cc = virtualInputs.cc.getArray();
+        this.oo.bcc = virtualInputs.bcc.getArray();
         this.oo.subject = $("subject").value;
         this.oo.priority = priority.getKey();
 
@@ -1634,6 +2051,10 @@ MailFunctions.prototype = {
         }
         $ALL(field + "_row").style.display = selected ? "" : "none";
         this.drawAttachmentPanel(true);
+    },
+    
+    additionalHeaderFieldIsVisible: function (field) {
+        return $ALL(field + "_row") && $ALL(field + "_row").style.display !== "none";
     }
 };
 
@@ -1720,7 +2141,6 @@ callback_new = function (cb) {
 initAddressBook = function () {
     // initialized?
     if (!myListAddressBook) {
-        var myFields = ["to", "cc", "bcc"];
         // initialize
         myListAddressBook = new SearchListAddressBook(
             null,
@@ -1771,10 +2191,11 @@ closeAddressbookNewEmail = function (ok) {
             }).get().join(", ");
             if (addresses.length !== 0) {
                 var value = jQuery("#" + suffix).attr("value");
-                if (value.length !== 0) {
-                    addresses = value + (value.match(/;|,[\s]?$/) ? "" : ", ") + addresses;
+                if (/\S/.test(value)) {
+                    addresses = value + (value.match(/[;,]\s*$/) ? "" : ", ") + addresses;
                 }
-                jQuery("#" + suffix).attr("value", addresses + ", ");
+                jQuery("#" + suffix).attr("value", addresses + ", ")
+                    .trigger('change');
                 value = null;
             }
             addresses = null;
