@@ -872,11 +872,15 @@ function switchtoView(newview, param) {
     // special case: configuration/folder|import does not change to configuration
     var isHybridConfig = /^configuration\/(folder|import)/.test(newview);
     if (ox.UIController && !isHybridConfig && newmodule !== ox.UIController.getModule()) {
-        // set module
-        ox.UIController.setModule({
-            module: newmodule,
-            success: cont
-        });
+    	if (/^configuration\//.test(newview)) {
+    		ox.UIController.configJump(newview, cont);
+        } else {
+	        // set module
+	        ox.UIController.setModule({
+	            module: newmodule,
+	            success: cont
+	        });
+        }
     } else {
         cont();
     }
