@@ -795,7 +795,10 @@ function updateTask() {
 				function finalFunction () {
 					function fn_cb(){
 						if (!reply.error) {
-							closeTaskWindow();
+						    if (!ox.api.window.isEmbedded() && ox.browser.Win8 && (ox.browser.IE10 || ox.browser.IE11)) {
+						        ox.api.window.core.triggerEvent("OX_Folder_Objects_Changed", activefolder);
+                            }
+						    closeTaskWindow();
 						}
 					}
 					// TODO: review bug fix 17278
@@ -878,7 +881,10 @@ function writeNewTask() {
 			function(reply) {
 				function finalFunction() {
 					function fn_cb(){
-						if (!reply.error) {					
+						if (!reply.error) {
+						    if (!ox.api.window.isEmbedded() && ox.browser.Win8 && (ox.browser.IE10 || ox.browser.IE11)) {
+						        ox.api.window.core.triggerEvent("OX_Folder_Objects_Changed", activefolder);
+						    }
 							closeTaskWindow();
 						}
 					}
