@@ -647,8 +647,20 @@ com_openexchange_subscribe.folderMenuItem.onShow = function() {
     ox.api.folder.get({
         folder: this.getContext().data.id,
         success: function (data) {
+            var visible = com_openexchange_subscribe.supportsFolder(data),
+            sources = com_openexchange_subscribe.sources;
+            // nothing to show?
+            for (var s in sources) {
+                if (data.module === sources[s].module) {
+                    visible = true;
+                    if (s.match(/.google.calendar$/) &&
+                            String(data.id) === String(ox.api.config.get('folder.calendar'))) {
+                        visible = false;
+                    }
+                }
+            }
             // support for this folder?
-            self.setVisible(com_openexchange_subscribe.supportsFolder(data));
+            self.setVisible(visible);
         }
     });
     return this.visible && this.enabled;
@@ -840,7 +852,7 @@ com_openexchange_subscribe.func_button_refresh_subscription_folder = function(mo
                                         if (reply.data[i][3].match(/.google.calendar$/)) {
                                             found = true;
                                             newConfirm(_("Warning"),
-                                                    _("Please note: Refreshing this scubscription will overwrite your changes you've made in this folder and replace it with the content of the external calendar! Do you really want to continue?"),
+                                                    _("Please note: Refreshing this subscription will overwrite your changes in this folder and replace them with the content of the external calendar. Do you really want to continue?"),
                                                     AlertPopup.YESNO, null, function() { join = lock = null; }, lock);
                                             break;
                                         }
@@ -887,21 +899,23 @@ com_openexchange_subscribe.func_button_refresh_subscription_folder = function(mo
     }
 };
 
-function refreshFolder(callback) {
-    ox.JSON.get(AjaxRoot + "/subscriptions?action=refresh&folder=" + activefolder + "&session=" + session,
+function refreshFolder(options) {
+    var opt = jQuery.extend({
+        folder: activefolder,
+        module: activemodule
+    }, options || {});
+    ox.JSON.get(AjaxRoot + "/subscriptions?action=refresh&folder=" + opt.folder +
+        (opt.id ? "&id=" + opt.id : "") + "&session=" + session,
         function() {
-            var module = activemodule;
-            if (module == "calendar") {
+            if (opt.module == "calendar") {
                 storageCache.update();
             } else {
-                OXCache.newRequest(null, activemodule,
-                    { criteria: { folder_id: activefolder } },
+                OXCache.newRequest(null, opt.module,
+                    { criteria: { folder_id: opt.folder } },
                     null, null, true);
             }
-            storageCache.update();
-            if(callback)
-            	callback();
-        });
+            if (opt.success) opt.success();
+        }, opt.error);
 }
 
 tab.init = function() {
@@ -923,9 +937,20 @@ tab.init = function() {
     
     var old_enter = temporary.folderTabs.enter;
     temporary.folderTabs.enter = function() {
-        tab.setVisible(Boolean(modules && this.folder.module in modules &&
-                               (this.folder.own_rights & (127 << 14)) >= (2 << 14)) &&
-                               	computePerm(this.folder.own_rights,0) >= 2);
+        var visible = (Boolean(modules && this.folder.module in modules &&
+                (this.folder.own_rights & (127 << 14)) >= (2 << 14)) &&
+                    computePerm(this.folder.own_rights,0) >= 2);
+        // nothing to show?
+        for (var s in sources) {
+            if (temporary.folderTabs.folder.module == sources[s].module) {
+                visible = true;
+                if (s.match(/.google.calendar$/) &&
+                        String(temporary.folderTabs.folder.id) === String(ox.api.config.get('folder.calendar'))) {
+                    visible = false;
+                }
+            }
+        }
+        tab.setVisible(visible);
         old_enter.call(this);
         if (sources) initSelect(); else onInit = initSelect;
     };
@@ -1000,23 +1025,6 @@ tab.init = function() {
 
     tab.load = function(cont) { load(selection.getSelected()[0], cont); };
     
-    function refresh(id, callback, error) {
-        ox.JSON.get(AjaxRoot + "/subscriptions?action=refresh&id=" + id +
-            "&folder=" + tab.parent.folder.id + "&session=" + session,
-            function() {
-                var module = tab.parent.folder.module;
-                if (module == "calendar") {
-                    storageCache.update();
-                } else {
-                    OXCache.newRequest(null, tab.parent.folder.module,
-                        { criteria: { folder_id: tab.parent.folder.id } },
-                        null, null, true);
-                }
-                callback();
-            }, error);
-    }
-    
-    
     tab.leave = function() {
     	// temporary fix for bug #16710
         // @todo: needs to be re-viewed by vp
@@ -1042,34 +1050,46 @@ tab.init = function() {
         if ("id" in data) {
             ox.JSON.put(AjaxRoot + "/subscriptions/?action=update&session=" +
                 session, data, function(reply) {
-                    refresh(data.id, function() {
-                        load(data.id, function(data) {
-                            storage.localUpdate([data.id], function() {
-                                return [data.id, data.displayName,
-                                        data.enabled];
-                            });
-                            savePending = false;
-                            cont(data);
-                        }, saveCancelled);
-                    }, saveCancelled);
+                    refreshFolder({
+                        id: data.id,
+                        folder: tab.parent.folder.id,
+                        module: tab.parent.folder.module,
+                        success: function() {
+                            load(data.id, function(data) {
+                                storage.localUpdate([data.id], function() {
+                                    return [data.id, data.displayName,
+                                            data.enabled];
+                                });
+                                savePending = false;
+                                cont(data);
+                            }, saveCancelled);
+                        },
+                        error: saveCancelled
+                    });
                 }, saveCancelled);
         } else {
             ox.JSON.put(AjaxRoot + "/subscriptions/?action=new&session=" +
                 session, data, function(reply) {
-                    refresh(reply.data, function() {
-                        load(reply.data, function(data) {
-                            storage.append([[data.id, data.displayName,
-                                             data.enabled]]);
-                            // update folder (local)
-                            ox.api.folder.update({
-                                folder: data.folder_id,
-                                local: true,
-                                data: { "com.openexchange.subscribe.subscriptionFlag": true }
-                            });
-                            savePending = false;
-                            cont(data);
-                        }, saveCancelled);
-                    }, saveCancelled);
+                    refreshFolder({
+                        id: reply.data,
+                        folder: tab.parent.folder.id,
+                        module: tab.parent.folder.module,
+                        success: function() {
+                            load(reply.data, function(data) {
+                                storage.append([[data.id, data.displayName,
+                                                 data.enabled]]);
+                                // update folder (local)
+                                ox.api.folder.update({
+                                    folder: data.folder_id,
+                                    local: true,
+                                    data: { "com.openexchange.subscribe.subscriptionFlag": true }
+                                });
+                                savePending = false;
+                                cont(data);
+                            }, saveCancelled);
+                        },
+                        error: saveCancelled
+                    });
                 }, saveCancelled);
         }
     };
@@ -1112,8 +1132,6 @@ tab.init = function() {
         ox.JSON.put(AjaxRoot + "/subscriptions?action=delete&session=" + session,
             ids, function(reply) { storage.removeIDs(ids); });
     }
-
-    
     
 
     // Right panel
@@ -1126,6 +1144,11 @@ tab.init = function() {
         for (var i in sources) {
             var m = sources[i].module;
             if (m == module) {
+                // ignore google subscription for default calendar folder
+                if (m === "calendar" && sources[i].id.match(/.google.calendar$/) &&
+                        String(tab.parent.folder.id) === String(ox.api.config.get('folder.calendar'))) {
+                    continue;
+                }
                 ids.push(i);
                 names.push(sources[i].displayName);
             }
@@ -1198,3 +1221,69 @@ tab.init = function() {
     tab.addWidget(subscriptiondur);
 };
 
+// OAuth autocreate hook
+temporary.services.provide("com.openexchange.oauth.autocreate", {
+    init: function(container) {
+        this.checkbox = new ox.UI.CheckBox(_("Subscribe in a new folder"));
+        this.checkbox.default_value = true;
+        container.addWidget(this.checkbox, "");
+    },
+    update: function(oauthid) {
+        var sources = com_openexchange_subscribe.sources;
+        for (var i in sources) {
+            var form = sources[i].formDescription;
+            if (form.length == 1 && form[0].widget == "oauthAccount" &&
+                form[0].options.type == oauthid)
+            {
+                this.source = sources[i];
+                this.checkbox.show();
+                return;
+            }
+        }
+        this.checkbox.hide();
+    },
+    save: function(data) {
+        if (!this.checkbox.get()) return;
+        var source = this.source;
+        var parent = configGetKey(["folder", source.module]);
+        ox.api.folder.getFreeTitle(parent, create, { title: data.displayName });
+        function create(title) {
+            ox.api.folder.create({
+                folder: parent,
+                data: {
+                    module: source.module,
+                    title: title
+                },
+                success: subscribe
+            });
+        }
+        function subscribe(folder) {
+            var subscription = {
+                folder: folder.id,
+                source: source.id
+            };
+            subscription[source.id] = {};
+            subscription[source.id][source.formDescription[0].name] = data.id;
+            ox.JSON.put(AjaxRoot + "/subscriptions/?action=new&session=" +
+                session, subscription, refreshSubscription);
+            function refreshSubscription(reply) {
+                refreshFolder({
+                    id: reply.data,
+                    folder: folder.id,
+                    module: folder.module,
+                    success: updateFolder
+                });
+            }
+            function updateFolder() {
+                // update folder (local)
+                ox.api.folder.update({
+                    folder: folder.id,
+                    local: true,
+                    data: {
+                        "com.openexchange.subscribe.subscriptionFlag": true
+                    }
+                });
+            }
+        }
+    }
+});
